function [center,radius] = LSsphere(data,varargin)
% LSSPHERE estimates a circumference or a sphere given its
% boundary points.
%
%   [center,radius] = LSsphere(data,varargin)
%
% MANDATORY INPUT
%
% data         : (double array) points to be estimated: 2D or 3D data.
%
% OPTIONAL INPUT
%
% nonlinearC         : (function handle) non linear constrain or 
%                                        the optimization
%
% OUTPUT
%
% center         : (double array) center of the estimated sphere
% radius         : (array) radius of the estimated sphere
%
%==========================================================================
%
% Version : 1.2 (06-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%



CONSTRAIN = 1e8;
mu        = 0;
w         = ones(length(data),1);

if (nargin-length(varargin)) ~= 1
    error('Wrong number of required parameters');
end

if (rem(length(varargin),2)==1)
    error('Optional parameters should always go by pairs');
    fprintf('!\n');
else
    for i=1:2:(length(varargin)-1)
        switch upper(varargin{i})
            case 'CONSTRAIN'
                CONSTRAIN = varargin{i+1};
            case 'MU'
                mu = varargin{i+1};
            case 'W'
                w = varargin{i+1};
                
        end
    end
end

nonlincon   = @(x) circcon(x,CONSTRAIN);

y           = (-sum(data.^2,2)).*sqrt(w);
R           = diag(sqrt(w)) * [-2*data, ones(size(data,1),1)];
mu          = 1/cond(R);
x0          = [mean(data) 0]';
fun         = @(x) 0.5*norm(R*x-y)^2+ mu/2*norm(x)^2;
options     = optimoptions(@fmincon,'Display','off');
center      = fmincon(fun,x0,[],[],[],[],[],[],nonlincon,options);
radius      = sqrt(sum(center(1:end-1).^2)-center(end));

center = center(1:end-1);